/* eslint-disable mocha/no-skipped-tests */
var linuxAutoUpdater = require('../../lib/autoUpdater/LinuxAutoUpdater'),
    _ = require('lodash'),
    hermes = require('@postman/hermes'),
    { expect } = require('chai'),
    sinon = require('sinon');

describe('linuxAutoUpdater', function () {
    let LinuxAutoUpdater,
        sandbox,
        options = {
            quitApp: _.noop,
            logger: {
                error: _.noop,
                info: _.noop,
                warn: _.noop
            },
            appInstallationPath: '/home/postman/Downloads/postman-app',
            appName: 'postman',
            originalFs: require('fs')
        };

    before(function () {
        sandbox = sinon.sandbox.create();
    });

    after(function () {
        sandbox.restore();
    });

    describe('downloadUpdate', function () {
        let downloadUpdateSpy;

        beforeEach(function () {
            LinuxAutoUpdater = new linuxAutoUpdater(options);
            downloadUpdateSpy = sandbox.spy(LinuxAutoUpdater, 'downloadUpdate');
        });

        afterEach(function () {
            downloadUpdateSpy.restore();
        });

        it('should throw an error if the updateInfo.appName is empty', () => {
            let err = null,
                appName = '',
                downloadURL = 'https://dl-beta.pstmn.io/download/version/6.7.0/linux_64',
                downloadDirectory = '/home/postman/Downloads/postman-app/';

            try {
                LinuxAutoUpdater.downloadUpdate(appName, downloadURL, downloadDirectory);
            }
            catch (e) {
                err = e;
            }
            finally {
                expect(downloadUpdateSpy.calledWith(appName, downloadURL, downloadDirectory));
                expect(err).to.be.instanceof(Error);
            }
        });

        it('should throw an error if the updateInfo.appName is not of type string', () => {
            let err = null,
                appName = 1,
                downloadURL = 'https://dl-beta.pstmn.io/download/version/6.7.0/linux_64',
                downloadDirectory = '/home/postman/Downloads/postman-app/';

            try {
                LinuxAutoUpdater.downloadUpdate(appName, downloadURL, downloadDirectory);
            }
            catch (e) {
                err = e;
            }
            finally {
                expect(downloadUpdateSpy.calledWith(appName, downloadURL, downloadDirectory));
                expect(err).to.be.instanceof(Error);
            }
        });

        it('should throw an error if the updateInfo.downloadURL is empty', (done) => {
            let appName = 'Postman',
                downloadURL = '',
                downloadDirectory = '/home/postman/Downloads/postman-app/',
                errorHandling = function (err) {
                    try {
                        expect(downloadUpdateSpy.calledWith(appName, downloadURL, downloadDirectory));
                        expect(err).to.be.instanceof(Error);
                        done();
                    }
                    catch (e) {
                        done(e);
                    }
                };

            LinuxAutoUpdater.on('error', errorHandling);
            LinuxAutoUpdater.downloadUpdate(appName, downloadURL, downloadDirectory);
        });

        it('should throw an error if the updateInfo.downloadURL is not of type string', (done) => {
            let appName = 'Postman',
                downloadURL = 8675,
                downloadDirectory = '/home/postman/Downloads/postman-app/',
                errorHandling = (err) => {
                    try {
                        expect(downloadUpdateSpy.calledWith(appName, downloadURL, downloadDirectory));
                        expect(err).to.be.instanceof(Error);
                        done();
                    }
                    catch (e) {
                        done(e);
                    }
                };

            LinuxAutoUpdater.on('error', errorHandling);
            LinuxAutoUpdater.downloadUpdate(appName, downloadURL, downloadDirectory);

        });

        it('should throw an error if the updateInfo.downloadDirectory is empty', (done) => {
            let appName = 'Postman',
                downloadURL = 'https://dl-beta.pstmn.io/download/version/6.7.0/linux_64',
                downloadDirectory = '',
                errorHandling = (err) => {
                    try {
                        expect(downloadUpdateSpy.calledWith(appName, downloadURL, downloadDirectory));
                        expect(err).to.be.instanceof(Error);
                        done();
                    }
                    catch (e) {
                        done(e);
                    }
                };

            LinuxAutoUpdater.on('error', errorHandling);
            LinuxAutoUpdater.downloadUpdate(appName, downloadURL, downloadDirectory);

        });

        it('should throw an error if the updateInfo.downloadDirectory is not of type string', (done) => {
            let appName = 'Postman',
                downloadURL = 'https://dl-beta.pstmn.io/download/version/6.7.0/linux_64',
                downloadDirectory = 1,
                errorHandling = (err) => {
                    try {
                        expect(downloadUpdateSpy.calledWith(appName, downloadURL, downloadDirectory));
                        expect(err).to.be.instanceof(Error);
                        done();
                    }
                    catch (e) {
                        done(e);
                    }
                };

            LinuxAutoUpdater.on('error', errorHandling);
            LinuxAutoUpdater.downloadUpdate(appName, downloadURL, downloadDirectory);
        });
    });

    describe('cleanUp', function () {
        let loggerSpy,
            hermesClearSpy;

        beforeEach(function () {
            LinuxAutoUpdater = new linuxAutoUpdater(options);
            loggerSpy = sandbox.spy(options.logger, 'info');
            hermesClearSpy = sandbox.spy(hermes, 'clear');
        });

        afterEach(function () {
            loggerSpy.restore();
            hermesClearSpy.restore();
        });

        it('should throw an error if options(argument) is empty', (done) => {
            let options = {},
                expectedError = 'LinuxAutoUpdater~cleanUp: Missing parameters, bailing out';

            LinuxAutoUpdater.cleanUp(options, (err) => {
                try {
                    expect(err.message).to.equal(expectedError);
                    expect(loggerSpy.callCount).to.equal(1);
                    expect(loggerSpy.calledWith(expectedError, options)).to.be.true;
                    expect(hermesClearSpy.callCount).to.equal(0);
                    done();
                }
                catch (e) {
                    done(e);
                }
            });
        });

        it('should throw an error if options have falsy values', (done) => {
            let options = false,
                expectedError = 'LinuxAutoUpdater~cleanUp: Missing parameters, bailing out';

            LinuxAutoUpdater.cleanUp(options, (err) => {
                try {
                    expect(err.message).to.equal(expectedError);
                    expect(loggerSpy.callCount).to.equal(1);
                    expect(loggerSpy.calledWith(expectedError, options)).to.be.true;
                    expect(hermesClearSpy.callCount).to.equal(0);
                    done();
                }
                catch (e) {
                    done(e);
                }
            });
        });

        it('should throw an error if options.appInstallationPath is empty', (done) => {
            let options = {
                    appName: 'Postman',
                    appInstallationPath: ''
                },
                expectedError = 'LinuxAutoUpdater~cleanUp: Missing parameters, bailing out';

            LinuxAutoUpdater.cleanUp(options, (err) => {
                try {
                    expect(err.message).to.equal(expectedError);
                    expect(loggerSpy.callCount).to.equal(1);
                    expect(loggerSpy.calledWith(expectedError, options)).to.be.true;
                    expect(hermesClearSpy.callCount).to.equal(0);
                    done();
                }
                catch (e) {
                    done(e);
                }
            });
        });

        it('should throw an error if options.appInstallationPath is not a string', (done) => {
            let options = {
                    appName: 'Postman',
                    appInstallationPath: 123
                },
                expectedError = 'InvalidArgumentException: Hermes~clear - clearInputData.directory should be of type string'; // eslint-disable-line max-len

            LinuxAutoUpdater.cleanUp(options, (err) => {
                try {
                    expect(err).to.be.instanceOf(Error);
                    expect(err.message).to.equal(expectedError);
                    expect(hermesClearSpy.called).to.be.true;
                    expect(hermesClearSpy.callCount).to.equal(1);
                    done();
                }
                catch (e) {
                    done(e);
                }
            });
        });

        it('should throw an error if options.appName is empty', (done) => {
            let options = {
                    appName: '',
                    appInstallationPath: '/home/postman/Downloads/postman-app/'
                },
                expectedError = 'LinuxAutoUpdater~cleanUp: Missing parameters, bailing out';

            LinuxAutoUpdater.cleanUp(options, (err) => {
                try {
                    expect(err.message).to.equal(expectedError);
                    expect(loggerSpy.callCount).to.equal(1);
                    expect(loggerSpy.calledWith(expectedError, options)).to.be.true;
                    expect(hermesClearSpy.callCount).to.equal(0);
                    done();
                }
                catch (e) {
                    done(e);
                }
            });
        });

        it('should throw an error if options.appName is not a string', (done) => {
            let options = {
                appName: 123,
                appInstallationPath: '/home/postman/Downloads/postman-app/'
            };

            LinuxAutoUpdater.cleanUp(options, (err) => {
                try {
                    expect(err).to.be.instanceOf(Error);
                    expect(hermesClearSpy.called).to.be.true;
                    expect(hermesClearSpy.callCount).to.equal(1);
                    done();
                }
                catch (e) {
                    done(e);
                }
            });
        });
    });

    describe('quitAndInstall', function () {
        let quitAppSpy,
            hermesSwapSpy;

        beforeEach(function () {
            LinuxAutoUpdater = new linuxAutoUpdater(options);
            quitAppSpy = sinon.spy(options, 'quitApp');
            hermesSwapSpy = sinon.spy(hermes, 'swap');
        });

        afterEach(function () {
            quitAppSpy.restore();
            hermesSwapSpy.restore();
        });

        it('should throw an error if updateInfo is empty', () => {
            let err = null,
                updateInfo = {};

            try {
                LinuxAutoUpdater.quitAndInstall(updateInfo);
            }
            catch (e) {
                err = e;
            }
            finally {
                expect(hermesSwapSpy.callCount).to.equal(1);
                expect(quitAppSpy.callCount).to.equal(0);
                expect(err).to.be.instanceof(Error);
            }
        });

        it('should throw an error if updateInfo is not an object', () => {
            let err = null,
                updateInfo = 'foo';

            try {
                LinuxAutoUpdater.quitAndInstall(updateInfo);
            }
            catch (e) {
                err = e;
            }
            finally {
                expect(hermesSwapSpy.callCount).to.equal(1);
                expect(quitAppSpy.callCount).to.equal(0);
                expect(err).to.be.instanceof(Error);
            }
        });

        it('should throw an error if updateInfo.eId is empty', () => {
            let err = null,
                updateInfo = {
                    eID: '',
                    packageName: 'somePackage',
                    downloadURL: 'http://sample.com/foo.tar.gz',
                    downloadChecksum: '2927b7c6ed350687446b7eba58bd74535afd411b',
                    downloadChecksumAlgorithm: 'sha1',
                    downloadInfo: {
                        status: 'DOWNLOAD_COMPLETED',
                        error: null,
                        downloadDirectory: 'User/PathForDownloadDirectory'
                    },
                    extractInfo: {
                        status: 'EXTRACTION_SUCCESSFUL',
                        error: null,
                        extractDirectory: 'User/PathForExtractDirectory'
                    }
                };

            try {
                LinuxAutoUpdater.quitAndInstall(updateInfo);
            }
            catch (e) {
                err = e;
            }
            finally {
                expect(hermesSwapSpy.callCount).to.equal(1);
                expect(quitAppSpy.callCount).to.equal(0);
                expect(err).to.be.instanceof(Error);
            }
        });

        it('should throw an error if updateInfo.packageName is empty', () => {
            let err = null,
                updateInfo = {
                    eID: 'somePackage-xxxxxxxxxxxxx',
                    packageName: '',
                    downloadURL: 'http://sample.com/foo.tar.gz',
                    downloadChecksum: '2927b7c6ed350687446b7eba58bd74535afd411b',
                    downloadChecksumAlgorithm: 'sha1',
                    downloadInfo: {
                        status: 'DOWNLOAD_COMPLETED',
                        error: null,
                        downloadDirectory: 'User/PathForDownloadDirectory'
                    },
                    extractInfo: {
                        status: 'EXTRACTION_SUCCESSFUL',
                        error: null,
                        extractDirectory: 'User/PathForExtractDirectory'
                    }
                };

            try {
                LinuxAutoUpdater.quitAndInstall(updateInfo);
            }
            catch (e) {
                err = e;
            }
            finally {
                expect(hermesSwapSpy.callCount).to.equal(1);
                expect(quitAppSpy.callCount).to.equal(0);
                expect(err).to.be.instanceof(Error);
            }
        });

        it('should throw an error if updateInfo.extractInfo is empty', () => {
            let err = null,
                updateInfo = {
                    eID: 'somePackage-xxxxxxxxxxxxx',
                    packageName: 'somePackage',
                    downloadURL: 'http://sample.com/foo.tar.gz',
                    downloadChecksum: '2927b7c6ed350687446b7eba58bd74535afd411b',
                    downloadChecksumAlgorithm: 'sha1',
                    downloadInfo: {
                        status: 'DOWNLOAD_COMPLETED',
                        error: null,
                        downloadDirectory: 'User/PathForDownloadDirectory'
                    },
                    extractInfo: {}
                };

            try {
                LinuxAutoUpdater.quitAndInstall(updateInfo);
            }
            catch (e) {
                err = e;
            }
            finally {
                expect(hermesSwapSpy.callCount).to.equal(1);
                expect(quitAppSpy.callCount).to.equal(0);
                expect(err).to.be.instanceof(Error);
            }
        });
    });
});
